import 'package:chariot/src/ioc/error.dart';

typedef Creator<T> = T Function(Ioc ioc);

class Ioc
{
    final Map<Type, Creator> _objects = {};
    final Map<Type, dynamic> _created = {};

    void set<T>(Creator<T> creator)
    {
        if(!has<T>()) {
            _objects[T] = creator;
        }
    }

    T get<T>({bool singleton = true})
    {
        if(!has<T>()) {
            throw UnregisteredType("Unregistered type '$T'");
        }

        if(!singleton || _created[T] == null) {
            return _created[T] = _objects[T](this);
        }

        return _created[T];
    }

    bool has<T>()
    {
        return _objects.containsKey(T);
    }

    void unset<T>()
    {
        _objects?.remove(T);
        _created?.remove(T);
    }
}